#ifndef HARKTOOL4_COMMONVARIABLES_HPP
#define HARKTOOL4_COMMONVARIABLES_HPP

#include <boost/program_options.hpp>
#include "Incrementer.hpp"

namespace harktool4
{

/** @brief よく使う変数を集めた便利構造体.
 * variables_mapから内容を読み取って構築する機能付き。
 */
struct CommonVariables
{
    const unsigned int nummic, numazi, numheight, numelv, cut_start, cut_end;
    const double azifrom, aziint, heightfrom, heightint, elvfrom, elvint, radius;
    const bool use_height;
    const unsigned int num_vertical;

    harktool4::Incrementer<double> azimuth, height, elevation;

    CommonVariables(const boost::program_options::variables_map& vm)
        : nummic(       vm["nummic"].as<unsigned int>()),
          numazi(       vm["numazi"].as<unsigned int>()),
          numheight(    vm["numheight"].as<unsigned int>()),
          numelv(       vm["numelv"].as<unsigned int>()),
          cut_start(    vm["cut-start"].as<unsigned int>()),
          cut_end(      vm["cut-end"].as<unsigned int>()),
          azifrom(      vm["azifrom"].as<double>()),
          aziint(       vm["aziint"].as<double>()),
          heightfrom(   vm["heightfrom"].as<double>()),
          heightint(    vm["heightint"].as<double>()),
          elvfrom(      vm["elvfrom"].as<double>()),
          elvint(       vm["elvint"].as<double>()),
          radius(       vm["radius"].as<double>()),
          use_height( // trueでheightを使い、falseでelevationを使う
                  !(vm["heightfrom"].defaulted() && vm["heightint"].defaulted() && vm["numheight"].defaulted()) ||
                  (vm["elvfrom"].defaulted() && vm["elvint"].defaulted() && vm["numelv"].defaulted())),
          num_vertical(use_height ? numheight : numelv),
          azimuth(azifrom, aziint, 0, 360),     // azifromを初期値に持つ、aziintずつ増加する増分器 下界は0, 上界は360
          height(heightfrom, heightint, -10000 ,10000),  // heightfromを初期値に持つ、heightintずつ増加する増分器 下界・上界はdoubleの限界値
          elevation(elvfrom, elvint, 0, 360)    // azifromを初期値に持つ、aziintずつ増加する増分器 下界は0, 上界は360
    {}

    CommonVariables& operator ++()
    {
        ++azimuth;
        ++height;
        ++elevation;
        return *this;
    }
};

}

#endif // HARKTOOL4_COMMONVARIABLES_HPP
